﻿Type=Class
Version=7.01
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
Sub Class_Globals
	Public sTypeName As String = "clsBranch"
	Private myID As String
	Private myName As String
	Private myItems As MapList ' ID is used as key
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize
	Dim r As Reflector
	r.Target = r.RunStaticMethod("java.util.UUID", "randomUUID", Null, Null)
	myID = r.RunMethod("toString")
	myName = ""
	myItems.Initialize
End Sub

'Property sID (readonly) - globally unique ID (aka UUID)
Public Sub getsID As String
	Return myID
End Sub

'Property sName - name of the Trunk
public Sub getsName As String
	Return myName
End Sub
public Sub setsName(NewVal As String)
	myName = NewVal
End Sub

'Property oItems - the items stored directly in the Branch
'NOTE: Items are expected to be clsLeaf ONLY
Public Sub getoItems As MapList
	Return myItems
End Sub

'Find an item by sID wherever it may be in the Trunk/Branch/Leaf hierarchy - returns null if not found
'sItemID is the ID of the instance to find
public Sub oFindByID(sItemId As String) As Object
	If sItemId.Length = 0 Then Return Null
	If myID = sItemId Then Return Me
	If myItems.ContainsKey(sItemId) Then Return myItems.Get(sItemId)
	Return Null
End Sub

'Find an item by sName wherever it may be in the Trunk/Branch/Leaf hierarchy (depth first search) - returns null if not found
'sItemName is the Name of the instance to find
'sTypeName is optionally the TypeName of the item to find
public Sub oFindByName(sItemName As String, sItemType As String) As Object
	If sItemName.Length = 0 Then Return Null
	If myName = sItemName Then
		If sItemType.Length = 0 Then Return Me
		If sItemType = sTypeName Then Return Me
		' If not the right type, keep looking
	End If
	For i = 0 To myItems.Size - 1
		Dim oItem As Object = myItems.GetValueAt(i)
		If oItem Is clsLeaf Then
			Dim oLeaf As clsLeaf = oItem
			If oLeaf.sName = sItemName Then
				If sItemType.Length = 0 Then Return oLeaf
				If sItemType = oLeaf.sTypeName Then Return oLeaf
				' keep looking
			End If
		End If
	Next
	Return Null
End Sub

'Remove a Leaf by sID - returns the deleted Leaf (or null if not found)
'sItemID is the ID of the Leaf to remove
public Sub oRemoveByID(sItemId As String) As Object
	If sItemId.Length = 0 Then Return Null
	If myItems.ContainsKey(sItemId) Then Return myItems.Remove(sItemId)
	Return Null
End Sub

'Insert a Leaf before another item in the Hierarchy - returns the inserted object (or null if it can't be inserted)
'sItemID is the sID of the item to find
'oInsertItem is the item to insert
'bInsertBefore is TRUE to insert before sItemID, FALSE to insert after sItemID
public Sub oInsertInHierarchy(sItemId As String, oInsertItem As Object, bInsertBefore As Boolean) As Object
	If sItemId.Length = 0 Then Return Null
	If myID = sItemId Then Return Null ' Should have been handled at Trunk level!
	If myItems.ContainsKey(sItemId) Then
		Dim sInsertKey As String
		If oInsertItem Is clsLeaf Then
			Dim oLeaf As clsLeaf = oInsertItem
			sInsertKey = oLeaf.sID
		Else
			Return Null ' Can only insert Leaf nodes
		End If
		If bInsertBefore Then
			myItems.InsertBefore(sItemId, sInsertKey, oInsertItem)
		Else
			myItems.InsertAfter(sItemId, sInsertKey, oInsertItem)
		End If
		Return myItems.Get(sInsertKey)
	End If
	Return Null
End Sub

'Insert a Leaf as a subitem of another item - returns the inserted object (or null if it can't be inserted)
'sParentID is the sID of the parent item to find
'oInsertItem is the item to insert
'bInsertAtStart is TRUE to insert at the start of the parent's subitems, false to add at the end
public Sub oInsertSubItem(sParentId As String, oInsertItem As Object, bInsertAtStart As Boolean) As Object
	If sParentId.Length = 0 Then Return Null
	Dim sInsertKey As String
	If oInsertItem Is clsLeaf Then
		Dim oLeaf As clsLeaf = oInsertItem
		sInsertKey = oLeaf.sID
	Else
		Return Null ' Can only insert Leaf nodes
	End If
	' Try myself
	If myID = sParentId Then
		If bInsertAtStart Then
			Return myItems.InsertAt(0, sInsertKey, oInsertItem)
		Else
			Return myItems.put(sInsertKey, oInsertItem)
		End If
	End If
	Return Null
End Sub

' Creates a MapList containing iteminfo entries for the trunk/branch/leaf hierarchy
public Sub toItemInfo(oItemIDs As MapList)
	' an entry for this branch
	Dim xBranch As tItemInfo: xBranch.Initialize
	xBranch.sID = myID
	xBranch.sName = myName
	xBranch.sTypeName = sTypeName
	xBranch.sParentID = ""
	xBranch.iPos = oItemIDs.Size
	oItemIDs.Put(xBranch.sID, xBranch)
	' process leaves
	For i = 0 To myItems.size - 1
		Dim oLeaf As clsLeaf = myItems.getvalueat(i)
		oLeaf.toItemInfo(oItemIDs, myID)
	Next
	' add an endgroup
	Dim xEndGroup As tItemInfo: xEndGroup.Initialize
	xEndGroup.sID = myID & "="
	xEndGroup.sName = Main.sENDGROUP
	xEndGroup.sTypeName = Main.sENDGROUP
	xEndGroup.sParentID = myID
	xEndGroup.iPos = oItemIDs.Size
	oItemIDs.Put(xEndGroup.sID, xEndGroup)
End Sub