﻿Type=Class
Version=7.01
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
Sub Class_Globals
	Public sTypeName As String = "clsTrunk"
	Private myID As String
	Private myName As String
	Private myItems As MapList ' ID is used as key
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize
	Dim r As Reflector
	r.Target = r.RunStaticMethod("java.util.UUID", "randomUUID", Null, Null)
	myID = r.RunMethod("toString")
	myName = ""
	myItems.Initialize
End Sub

'Property sID (readonly) - globally unique ID (aka UUID)
Public Sub getsID As String
	Return myID
End Sub

'Property sName - name of the Trunk
public Sub getsName As String
	Return myName
End Sub
public Sub setsName(NewVal As String)
	myName = NewVal
End Sub

' Property oItems - the items stored directly in the Trunk
'NOTE: Items are expected to be either clsBranch or clsLeaf
Public Sub getoItems As MapList
	Return myItems
End Sub

'Find an item by sID wherever it may be in the Trunk/Branch/Leaf hierarchy - returns null if not found
'sItemID is the sID of the instance to find
public Sub oFindByID(sItemId As String) As Object
	If sItemId.Length = 0 Then Return Null
	If myID = sItemId Then Return Me
	If myItems.ContainsKey(sItemId) Then Return myItems.Get(sItemId) ' ID was within Trunk items
	For i = 0 To myItems.Size - 1
		Dim oItem As Object = myItems.GetValueAt(i)
		If oItem Is clsBranch Then
			Dim oBranch As clsBranch = oItem
			oItem = oBranch.oFindByID(sItemId)
			If Not(oItem = Null) Then Return oItem ' ID was within Branch items
			' keep looking
		End If
	Next
	Return Null
End Sub

'Find an item by sName wherever it may be in the Trunk/Branch/Leaf hierarchy (depth first search) - returns null if not found
'sItemName is the Name of the instance to find
'sTypeName is optionally the TypeName of the item to find
public Sub oFindByName(sItemName As String, sItemType As String) As Object
	If sItemName.Length = 0 Then Return Null
	If myName = sItemName Then
		If sItemType.Length = 0 Then Return Me
		If sItemType = sTypeName Then Return Me
		' If not the right type, keep looking
	End If
	For i = 0 To myItems.Size - 1
		Dim oItem As Object = myItems.GetValueAt(i)
		If oItem Is clsLeaf Then
			Dim oLeaf As clsLeaf = oItem
			If oLeaf.sName = sItemName Then
				If sItemType.Length = 0 Then Return oLeaf
				If sItemType = oLeaf.sTypeName Then Return oLeaf
				' keep looking
			End If
		Else If oItem Is clsBranch Then
			Dim oBranch As clsBranch = oItem
			If oBranch.sName = sItemName Then
				If sItemType.Length = 0 Then Return oBranch
				If sItemType = oBranch.sTypeName Then Return oBranch
				' Try finding within the branch (depth first)
				oItem = oBranch.oFindByName(sItemName, sItemType)
				If Not(oItem = Null) Then Return oItem
				' keep looking
			End If
		End If
	Next
	Return Null
End Sub

'Remove a Branch or Leaf by sID wherever it may be in the hierarchy - returns the deleted object (or null if not found)
'sItemID is the sID of the instance to remove
public Sub oRemoveByID(sItemId As String) As Object
	If sItemId.Length = 0 Then Return Null
	If myItems.ContainsKey(sItemId) Then Return myItems.Remove(sItemId)
	For i = 0 To myItems.Size - 1
		Dim oItem As Object = myItems.GetValueAt(i)
		If oItem Is clsBranch Then
			Dim oBranch As clsBranch = oItem
			oItem = oBranch.oRemoveByID(sItemId) 
			If Not(oItem = Null) Then Return oItem
		End If
	Next
	Return Null
End Sub

'Insert a Branch or Leaf before or after another item in the Hierarchy - returns the inserted object (or null if it can't be inserted)
'sItemID is the sID of the item to find
'oInsertItem is the item to insert
'bInsertBefore is TRUE to insert before sItemID, FALSE to insert after sItemID
public Sub oInsertInHierarchy(sItemId As String, oInsertItem As Object, bInsertBefore As Boolean) As Object
	If sItemId.Length = 0 Then Return Null
	If myID = sItemId Then Return Null ' Can't insert before/after the Trunk item
	If myItems.ContainsKey(sItemId) Then
		Dim sInsertKey As String
		If oInsertItem Is clsLeaf Then
			Dim oLeaf As clsLeaf = oInsertItem
			sInsertKey = oLeaf.sID
		Else If oInsertItem Is clsBranch Then
			Dim oBranch As clsBranch = oInsertItem
			sInsertKey = oBranch.sID
		Else
			Return Null ' Can only insert Leaf or Branch nodes
		End If
		If bInsertBefore Then
			myItems.InsertBefore(sItemId, sInsertKey, oInsertItem)
		Else
			myItems.InsertAfter(sItemId, sInsertKey, oInsertItem)
		End If
		Return myItems.Get(sInsertKey)
	End If
	For i = 0 To myItems.Size - 1
		Dim oItem As Object = myItems.GetValueAt(i)
		If oItem Is clsBranch Then
			Dim oBranch As clsBranch = oItem
			oItem = oBranch.oInsertInHierarchy(sItemId, oInsertItem, bInsertBefore)
			If Not(oItem = Null) Then Return oItem ' found and inserted the item, else try next branch
		End If
	Next
	Return Null
End Sub

'Insert a Branch or Leaf as a subitem of another item - returns the inserted object (or null if it can't be inserted)
'sParentID is the sID of the parent item to find
'oInsertItem is the item to insert
'bInsertAtStart is TRUE to insert at the start of the parent's subitems, false to add at the end
public Sub oInsertSubItem(sParentId As String, oInsertItem As Object, bInsertAtStart As Boolean) As Object
	If sParentId.Length = 0 Then Return Null
	Dim sInsertKey As String
	If oInsertItem Is clsLeaf Then
		Dim oLeaf As clsLeaf = oInsertItem
		sInsertKey = oLeaf.sID
	Else If oInsertItem Is clsBranch Then
		Dim oBranch As clsBranch = oInsertItem
		sInsertKey = oBranch.sID
	Else
		Return Null ' Can only insert Leaf or Branch nodes
	End If
	' Try myself (trunk) first
	If myID = sParentId Then
		If bInsertAtStart Then
			Return myItems.InsertAt(0, sInsertKey, oInsertItem)
		Else
			Return myItems.put(sInsertKey, oInsertItem)
		End If
	End If
	' If not me, then only other parents are the branches in myItems
	If myItems.ContainsKey(sParentId) Then
		Dim oBranch As clsBranch = myItems.Get(sParentId)
		Return oBranch.oInsertSubItem(sParentId, oInsertItem, bInsertAtStart)
	End If
	Return Null
End Sub

public Sub SetupTestData
	Dim dbSQL As SQL
	'Initializes the database
	If Not(File.Exists(File.DirInternal, "citylist.db")) Then
		File.Copy(File.DirAssets, "cities.db", File.DirInternal, "citylist.db")
	End If
	
	dbSQL.Initialize(File.DirInternal, "citylist.db", False)

	'Creates a cursor on data
	Dim dbCursor As Cursor = dbSQL.ExecQuery("SELECT City, Country FROM Cities where country in (SELECT Country FROM Cities group by country having count(*) >= 3 and count(*) <= 8) ORDER BY Country, City")

	'Builds the city & country list
	Dim Country, City As String
	myItems.Initialize
	For i = 0 To dbCursor.RowCount - 1
		dbCursor.Position = i
		Country = dbCursor.GetString("Country")
		City = dbCursor.GetString("City")
		'Get/setup the country
		Dim oBranch As clsBranch: oBranch.Initialize
		Dim oItem As Object = oFindByName(Country, oBranch.sTypeName)
		If oItem = Null Then
			oBranch.sName = Country
			myItems.Put(oBranch.sID, oBranch)
		Else
			oBranch = oItem
		End If
		'Get/setup the city
		Dim oLeaf As clsLeaf: oLeaf.Initialize
		Dim oItem As Object = oBranch.oFindByName(City, oLeaf.sTypeName)
		If oItem = Null Then
			oLeaf.sName = City
			oBranch.oItems.Put(oLeaf.sID, oLeaf)
		End If
	Next

	'When the user quits, the database is deleted
	If File.Exists(File.DirInternal, "citylist.db") Then
		File.Delete(File.DirInternal, "citylist.db")
	End If

End Sub

' Creates a MapList containing iteminfo entries for the trunk/brnch/leaf hierarchy
public Sub toItemInfo As MapList
	Dim oItemIDs As MapList: oItemIDs.Initialize
	' there's no entry for the trunk
	' process branches/leaves
	For i = 0 To myItems.size - 1
		Dim oItem As Object = myItems.getvalueat(i)
		If oItem Is clsLeaf Then
			Dim oLeaf As clsLeaf = oItem
			oLeaf.toItemInfo(oItemIDs, "")
		else If oItem Is clsBranch Then
			Dim oBranch As clsBranch = oItem
			oBranch.toItemInfo(oItemIDs)
		End If
	Next
	Return oItemIDs
End Sub